﻿using System;
using System.Windows.Forms;
using System.IO.Ports;
using System.Drawing;
using System.Text;
using System.Media;

namespace ComPortSniffer
{
   public partial class Form1 : Form
   {
      SerialPort port1;
      SerialPort port2;
      readonly Text tbText = new Text();

      const String selectPorts = "Com-Ports nicht ausgewählt";
      public Form1()
      {
         InitializeComponent();
      }

      private void Form1_Load(object sender, EventArgs e)
      {
         cbComportSelector1.TextChanged += new System.EventHandler(this.cbComportSelector_TextChanged);
         cbComportSelector2.TextChanged += new System.EventHandler(this.cbComportSelector_TextChanged);
         tbLog.TextChanged += TbLog_TextChanged;

         cbComportSelector1.Items.AddRange(ComDescriptor.getPorts(true).ToArray());
         cbComportSelector1.SelectedIndex = 0;
         cbComportSelector2.Items.AddRange(ComDescriptor.getPorts(false).ToArray());
         cbComportSelector2.SelectedIndex = 0;

         cbBaudrateSelector.SelectedIndex = 6;
         cbDataBitsSelector.SelectedIndex = 3;
         cbStopBitsSelector.SelectedIndex = 1;
         cbParitySelector.SelectedIndex = 0;

         cmdStop.Left = cmdStart.Left;
         cmdStop.Top = cmdStart.Top;

      //   StatusLabel.Text = selectPorts;

         tbLog.setRtf(tbText.getRTF(cbHexDisplay.Checked));
      }

      private void TbLog_TextChanged(object sender, EventArgs e)
      {
         if (cbAutoScoll.Checked)
         {
            var start = tbLog.SelectionStart;
            var length = tbLog.SelectionLength;
            tbLog.SelectionStart = tbLog.Text.Length;
            tbLog.ScrollToCaret();
            tbLog.SelectionStart = start;
            tbLog.SelectionLength = length;
         }
      }

      private void cbComportSelector1_DropDown(object sender, EventArgs e)
      {
         cbComportSelector1.Items.Clear();
         cbComportSelector1.Items.AddRange(ComDescriptor.getPorts(true).ToArray());
      }

      private void cbComportSelector2_DropDown(object sender, EventArgs e)
      {
         cbComportSelector2.Items.Clear();
         cbComportSelector2.Items.AddRange(ComDescriptor.getPorts(false).ToArray());
      }

      private void cbComportSelector_TextChanged(object sender, EventArgs e)
      {

         if (cbComportSelector1.SelectedIndex !=-1 && cbComportSelector2.SelectedIndex !=-1)
         {
            StatusLabel.Text = "";
            cmdStart.Enabled = true;
         }
         else
         {
            StatusLabel.Text = selectPorts;
            cmdStart.Enabled = false;
         }

         String caption = "URS ComPort-Sniffer ";
         if (cbComportSelector1.SelectedItem != null)
            caption += ((ComDescriptor)cbComportSelector1.SelectedItem).PortName;
         else
            caption += "?";
         caption += " <-> ";
         if (cbComportSelector2.SelectedItem != null)
            caption += ((ComDescriptor)cbComportSelector2.SelectedItem).PortName;
         else
            caption += "?";

         Text = caption;
      }

      private void cmdStart_Click(object sender, EventArgs e)
      {
         var baud = cbParitySelector.SelectItem;
         var datbits = (int)cbDataBitsSelector.SelectItem;
         var p = (ComDescriptor)cbComportSelector1.SelectedItem;
         port1 = new SerialPort(p.PortName,
                                (int)cbBaudrateSelector.SelectItem,
                                (Parity)cbParitySelector.SelectItem,
                                (int)cbDataBitsSelector.SelectItem,
                                (System.IO.Ports.StopBits)cbStopBitsSelector.SelectItem);
         p = (ComDescriptor)cbComportSelector2.SelectedItem;
         port2 = new SerialPort(p.PortName,
                                (int)cbBaudrateSelector.SelectItem,
                                (Parity)cbParitySelector.SelectItem,
                                (int)cbDataBitsSelector.SelectItem,
                                (System.IO.Ports.StopBits)cbStopBitsSelector.SelectItem);
         port1.DataReceived += Port1_DataReceived;
         port2.DataReceived += Port2_DataReceived;

         port1.PinChanged += Port1_PinChanged;
         port2.PinChanged += Port2_PinChanged;

         try
         {
            port1.Open();
         }
         catch
         {
            StatusLabel.Text = "Port " + port1.PortName + " kann nicht geöffnet werden.";
            SystemSounds.Beep.Play();
            return;
         }

         try
         {
            port2.Open();
         }
         catch
         {
            StatusLabel.Text = "Port " + port2.PortName + " kann nicht geöffnet werden.";
            port1.Close();
            SystemSounds.Beep.Play();
            return;
         }

         while (port1.BytesToRead > 0)
            port1.ReadByte();
         while (port2.BytesToRead > 0)
            port2.ReadByte();

         port1.DtrEnable = port2.DsrHolding;
         port1.RtsEnable = port2.CtsHolding;
         port2.DtrEnable = port1.DsrHolding;
         port2.RtsEnable = port1.CtsHolding;

         tbLog.Clear();

         cmdStart.Visible = false;
         cmdStop.Visible = true;

         cbComportSelector1.Enabled = false;
         cbComportSelector2.Enabled = false;
         cbBaudrateSelector.Enabled = false;
         cbDataBitsSelector.Enabled = false;
         cbParitySelector.Enabled = false;
         cbStopBitsSelector.Enabled = false;

         tbLog.BackColor = SystemColors.Control;
         StatusLabel.Text = "Sniffer bereit";
      }

      private void cmdStop_Click(object sender, EventArgs e)
      {
         port1.Close();
         port2.Close();
         cmdStart.Visible = true;
         cmdStop.Visible = false;

         cbComportSelector1.Enabled = true;
         cbComportSelector2.Enabled = true;
         cbBaudrateSelector.Enabled = true;
         cbDataBitsSelector.Enabled = true;
         cbParitySelector.Enabled = true;
         cbStopBitsSelector.Enabled = true;
         tbLog.BackColor = SystemColors.ControlLight;
         StatusLabel.Text = "";
      }


      private void Port1_DataReceived(object sender, SerialDataReceivedEventArgs e)
      {
         int btr;
         while ((btr = port1.BytesToRead) > 0)
         {
            byte[] buffer = new byte[btr];

            port1.Read(buffer, 0, btr);
            port2.Write(buffer, 0, btr);

            tbText.add(buffer, TextBlockColors.Port1);
            tbLog.setRtf(tbText.getRTF(cbHexDisplay.Checked));
         }
      }

      private void Port2_DataReceived(object sender, SerialDataReceivedEventArgs e)
      {
         int btr;
         while ((btr = port2.BytesToRead) > 0)
         {
            byte[] buffer = new byte[btr];

            port2.Read(buffer, 0, btr);
            port1.Write(buffer, 0, btr);

            tbText.add(buffer, TextBlockColors.Port2);
            tbLog.setRtf(tbText.getRTF(cbHexDisplay.Checked));
         }
      }

      private void Port1_PinChanged(object sender, SerialPinChangedEventArgs e)
      {
         if (e.EventType == SerialPinChange.DsrChanged)
            port2.DtrEnable = port1.DsrHolding;
         else if (e.EventType == SerialPinChange.CtsChanged)
            port2.RtsEnable = port1.CtsHolding;
      }

      private void Port2_PinChanged(object sender, SerialPinChangedEventArgs e)
      {
         if (e.EventType == SerialPinChange.DsrChanged)
            port1.DtrEnable = port2.DsrHolding;
         else if (e.EventType == SerialPinChange.CtsChanged)
            port1.RtsEnable = port2.CtsHolding;
      }


      private void cbHexDisplay_CheckedChanged(object sender, EventArgs e)
      {
         tbLog.setRtf(tbText.getRTF(cbHexDisplay.Checked));
      }

      private void cmdClear_Click(object sender, EventArgs e)
      {
         tbLog.Clear();
      }

      private void cmdAbout_Click(object sender, EventArgs e)
      {
         var a = new AboutBox();
         a.ShowDialog();
      }
   } // class
} // Namespace